var productSn = 'pt11ac75-b767-436d-465d-fd786d9958b6';
var apiUrl    = 'http://api.lm.234268.xyz/v1';
var time      = getTime();
var manifest  = getManifest();
var clientId  = getClientId();
var version   = manifest.version;
var isDebug   = getIsDebug();
var platform  = getPlatform();
var defaultLang;
var clientIP;
var configStr;
var configData;
var accessToken;
var proxyUrls;
var pacScriptStr;
var proxyConfig;
var switchStatus;
var usageDuration;
var email;
var userLevel;

proxyUrls = [];

function getPlatform() {
    var userAgent = navigator.userAgent;
    if (userAgent.indexOf('Chrome') !== -1 && userAgent.indexOf('Edge') === -1) {
        platform = 'chrome';
    }
    if (userAgent.indexOf('Edg') !== -1) {
        platform = 'edge';
    }
    if (userAgent.indexOf('Firefox') !== -1) {
        platform = 'firefox';
    }
    if (!platform) {
        platform = 'unknown';
    }
    chrome.storage.local.set({'platform': platform});
}

function getTime() {
    var date = new Date();
    return date.getTime() / 1000;
}

function getClientId() {
    return chrome.runtime.id;
}

function getClientLang() {
    return window.navigator.language;
}

function getManifest() {
    return chrome.runtime.getManifest();
}

function setApiUrl() {
    if (configData.data.api.url) {
        apiUrl = configData.data.api.url;
    }
}

function setUsageDuration() {
    if (configData.usage_duration) {
        usageDuration = configData.usage_duration;
        if (usageDuration > 0) {
            usageDuration = usageDuration * 1000;
        } else {
            usageDuration = 60 * 1.5 * 1000;
        }
        if (isDebug == 'true') {
            console.log('usageDuration->' + usageDuration);
        }
    }
}

function getIsDebug() {
    chrome.storage.local.get(['isDebug'], function(result) {
        return result.isDebug;
    });
}

function getLanguage() {
    return new Promise((resolve, reject) => {
        chrome.i18n.getAcceptLanguages(function(languages) {
            defaultLang = languages[0];
            chrome.storage.local.set({'language': defaultLang});
            if (isDebug == 'true') {
                console.log('Languages->' + languages);
            }
            resolve(defaultLang);
        });
    });
}

chrome.runtime.onInstalled.addListener(getPlatform);
chrome.runtime.onStartup.addListener(getPlatform);
chrome.runtime.onInstalled.addListener(getLanguage);
chrome.runtime.onStartup.addListener(getLanguage);

async function getClientIP() {
    const response = await fetch(apiUrl + '/services/opens/ips/format/json');
    const data = await response.json();
    clientIP = data.data.ip;
    if (isDebug == 'true') {
        console.log('Background clientIP->' + clientIP);
    }
    return clientIP;
}
//clientIP = getClientIP();

function getConfigUpdateTimeDuration() {
    var configStr;
    chrome.storage.local.get(['config'], function(result) {
        configStr = result.config;
    });
    var configData = JSON.parse(configStr || null);
    var configUpdateTime = configData.update_time;
    var configDuration = configData.request_duration;
    return { configUpdateTime, configDuration };
}

function getNodeUpdateTimeDuration() {
    var nodeStr;
    chrome.storage.local.get(['node'], function(result) {
        nodeStr = result.node;
    });
    var nodeData = JSON.parse(nodeStr || null);
    var nodeUpdateTime = nodeData.update_time;
    var nodeDuration = nodeData.request_duration;
    return { nodeUpdateTime, nodeDuration };
}

function getLoginStatus() {
    var domain = configData.data.base.root_url;
    chrome.cookies.get({url: 'http://' + domain, name: 'email'}, function(cookie) {
        if (cookie) {
            var email = cookie.value;
        }
        if(email){
            chrome.storage.local.set({'isLogin': 'yes'});
            var accessToken;
            chrome.storage.local.get(['accessToken'], function(result) {
                accessToken = result.accessToken;
                if (!accessToken) {
                    getAccessToken(email);
                }
            });
        } else {
            chrome.storage.local.set({'isLogin': 'no'});
        }
    });
}

async function getAccessToken(email) {
    try {
        var paramStr;
        var accessTokenTemp;
        chrome.storage.local.get(['accessToken'], function(result) {
            accessTokenTemp = result.accessToken;
        });
        var userLevelTemp;
        chrome.storage.local.get(['userLevel'], function(result) {
            userLevelTemp = result.userLevel;
        });
        if (email) {
            paramStr = '/email/' + email;
        }
        if (accessTokenTemp && userLevelTemp) {
            paramStr = paramStr + '/access_token/' + accessTokenTemp + '/user_level/' + userLevelTemp;
        }
        var requestUrl = apiUrl + '/services/opens/access_tokens/platform/' + platform + '/product_sn/' + productSn + '/version_id/' + version + paramStr + '/format/json';
        const response = await fetch(requestUrl);
        const data = await response.json();

        var res = data.data;
        accessToken = res.access_token;
        userLevel = res.user_level;

        chrome.storage.local.set({'accessToken': accessToken});
        chrome.storage.local.set({'userLevel': userLevel});

        if (isDebug == 'true') {
            console.log('Get access token succeeded');
            console.log(accessToken);
        }

        return res;
    } catch (error) {
        if (isDebug == 'true') {
            console.log('Failed to get access token');
        }

        return error;
    }
}

async function getConfig() {
    try {
        const response = await fetch(apiUrl + '/services/opens/configs/platform/' + platform + '/product_sn/' + productSn + '/version_id/' + version + '/format/json');
        const data = await response.json();
        configStr = JSON.stringify(data);

        chrome.storage.local.set({'config': configStr}, function() {
            if (isDebug == 'true') {
                console.log('Config saved');
            }
        });

        configData = res;
        proxyUrls = configData['data']['base']['match'];

        setApiUrl();
        getLoginStatus();
        setUsageDuration();

        if (isDebug == 'true') {
            console.log('Get configuration succeeded');
            console.log(configData);
            console.log(proxyUrls);
        }
    } catch (error) {
        chrome.storage.local.get(['config'], function(result) {
            configStr = result.config;
        });
        configData = JSON.parse(configStr || null);
        proxyUrls = configData['data']['base']['match'];

        setApiUrl();
        getLoginStatus();
        setUsageDuration();

        if (isDebug == 'true') {
            console.log('Failed to get configuration');
            console.log(configData);
            console.log(proxyUrls);
        }
    }
}
getConfig();

async function getNode() {
    try {
        const response = await fetch(apiUrl + '/services/opens/node_lists/product_sn/' + productSn + '/format/json');
        const data = await response.json();

        var nodeStr = JSON.stringify(data);

        chrome.storage.local.set({'node': nodeStr});
    } catch (error) {
        //console.log('Failed to get node');
    }
}
getNode();

chrome.storage.local.set({'productSn': productSn});

if (isDebug == 'true') {
    console.log(clientId);
    console.log(version);
}

chrome.runtime.onMessage.addListener(function(request, sender, sendResponse) {
    if (request.type === 'toggle-proxy') {
        var status = request.status;
        if (isDebug == 'true') {
            console.log('onMessage - Start: ', status);
        }
        chrome.storage.local.get(['switchStatus', 'pacScript'], function({switchStatus, pacScript}) {
            const pacInfo = atob(pacScript);
            const proxyInfo = pacInfo && pacInfo.split(/:|\s+/);
            const protocol = proxyInfo && proxyInfo[0].trim();
            const ip = proxyInfo && proxyInfo[1].trim();
            const port = proxyInfo && proxyInfo[2].trim();
            const proxyRules = proxyInfo ? [{
                id: 'proxyRule',
                priority: 1,
                action: {
                    type: 'modifyHeaders',
                    requestHeaders: [
                        {
                            header: 'Proxy-Connection',
                            operation: 'remove'
                        }
                    ]
                },
                condition: {
                    regexFilter: `.*`,
                    domains: proxyUrls.map((url) => url.replace(/(^\*\.)|(^.+)/, '')),
                    urlFilter: proxyUrls.map((url) => url + '*'),
                }
            },{
                id: 'proxyRule2',
                priority: 2,
                action: {
                    type: 'redirectToProxy',
                    proxyInfo: {
                        host: ip,
                        port: parseInt(port),
                        scheme: protocol,
                        username: '',
                        password: '',
                        socksUsername: '',
                        socksPassword: '',
                    }
                },
                condition: {
                    regexFilter: `.*`,
                    domains: proxyUrls.map((url) => url.replace(/(^\*\.)|(^.+)/, '')),
                    urlFilter: proxyUrls.map((url) => url + '*'),
                }
            }] : [];
            const directRule = {
                id: 'directRule',
                priority: 3,
                action: {
                    type: 'cancelRequest',
                },
                condition: {
                    regexFilter: `.*`,
                    urlFilter: ['<all_urls>'],
                        unless: {
                        domains: proxyUrls.map((url) => url.replace(/(^\*\.)|(^.+)/, '')),
                    }
                }
            };
            const rules = switchStatus === 'on' ? [...proxyRules, directRule] : [directRule];
            pacScriptStr = "function FindProxyForURL(url, host) {\n" +
                "    var proxyUrls = " + JSON.stringify(proxyUrls) + ";\n" +
                "    for (var i = 0; i < proxyUrls.length; i++) {\n" +
                "        if (shExpMatch(url, proxyUrls[i] + '*')) {\n" +
                "            return '" + protocol + " " + ip + ":" + port + "; DIRECT';\n" +
                "        }\n" +
                "    }\n" +
                "    console.log('Not using proxy for ' + url);\n" +
                "    return 'DIRECT';\n" +
                "}";
            chrome.proxy.settings.set({
                value: {
                    mode: 'pac_script',
                    pacScript: {
                        data: pacScriptStr
                    }
                },
                scope: 'regular'
            }, function() {
                chrome.declarativeNetRequest.onRequest && chrome.declarativeNetRequest.onRequest.addRules(rules, function() {
                    if (chrome.runtime.lastError) {
                        console.error('Error adding rules: ' + chrome.runtime.lastError.message);
                    } else {
                        if (isDebug == 'true') {
                            console.log('onBeforeRequest - Listener Added');
                        }
                    }
                });
            });
        });
    }
});

